@@ -24,10 +24,10 @@
 template<typename elem_type, typename derived>
 struct Base_extra_yes
   {
-  inline arma_warn_unused const Op<derived,op_inv_gen_default> i() const;   //!< matrix inverse
+  arma_warn_unused inline const Op<derived,op_inv_gen_default> i() const;   //!< matrix inverse
   
-  inline arma_warn_unused bool is_sympd() const;
-  inline arma_warn_unused bool is_sympd(typename get_pod_type<elem_type>::result tol) const;
+  arma_warn_unused inline bool is_sympd() const;
+  arma_warn_unused inline bool is_sympd(typename get_pod_type<elem_type>::result tol) const;
   };
 
 
@@ -51,14 +51,14 @@ struct Base_extra<elem_type, derived, false> { typedef Base_extra_no<elem_type,
 template<typename elem_type, typename derived>
 struct Base_eval_Mat
   {
-  arma_inline arma_warn_unused const derived& eval() const;
+  arma_warn_unused arma_inline const derived& eval() const;
   };
 
 
 template<typename elem_type, typename derived>
 struct Base_eval_expr
   {
-  inline arma_warn_unused Mat<elem_type> eval() const;   //!< force the immediate evaluation of a delayed expression
+  arma_warn_unused inline Mat<elem_type> eval() const;   //!< force the immediate evaluation of a delayed expression
   };
 
 
@@ -76,18 +76,18 @@ struct Base_eval<elem_type, derived, false> { typedef Base_eval_expr<elem_type,
 template<typename derived>
 struct Base_trans_cx
   {
-  arma_inline arma_warn_unused const Op<derived,op_htrans>  t() const;
-  arma_inline arma_warn_unused const Op<derived,op_htrans> ht() const;
-  arma_inline arma_warn_unused const Op<derived,op_strans> st() const;  // simple transpose: no complex conjugates
+  arma_warn_unused arma_inline const Op<derived,op_htrans>  t() const;
+  arma_warn_unused arma_inline const Op<derived,op_htrans> ht() const;
+  arma_warn_unused arma_inline const Op<derived,op_strans> st() const;  // simple transpose: no complex conjugates
   };
 
 
 template<typename derived>
 struct Base_trans_default
   {
-  arma_inline arma_warn_unused const Op<derived,op_htrans>  t() const;
-  arma_inline arma_warn_unused const Op<derived,op_htrans> ht() const;
-  arma_inline arma_warn_unused const Op<derived,op_htrans> st() const;  // return op_htrans instead of op_strans, as it's handled better by matrix multiplication code
+  arma_warn_unused arma_inline const Op<derived,op_htrans>  t() const;
+  arma_warn_unused arma_inline const Op<derived,op_htrans> ht() const;
+  arma_warn_unused arma_inline const Op<derived,op_htrans> st() const;  // return op_htrans instead of op_strans, as it's handled better by matrix multiplication code
   };
 
 
@@ -124,8 +124,8 @@ struct Base
   arma_cold inline void brief_print(                           const std::string extra_text = "") const;
   arma_cold inline void brief_print(std::ostream& user_stream, const std::string extra_text = "") const;
   
-  inline arma_warn_unused elem_type min() const;
-  inline arma_warn_unused elem_type max() const;
+  arma_warn_unused inline elem_type min() const;
+  arma_warn_unused inline elem_type max() const;
   
   inline elem_type min(uword& index_of_min_val) const;
   inline elem_type max(uword& index_of_max_val) const;
@@ -133,31 +133,33 @@ struct Base
   inline elem_type min(uword& row_of_min_val, uword& col_of_min_val) const;
   inline elem_type max(uword& row_of_max_val, uword& col_of_max_val) const;
   
-  inline arma_warn_unused uword index_min() const;
-  inline arma_warn_unused uword index_max() const;
+  arma_warn_unused inline uword index_min() const;
+  arma_warn_unused inline uword index_max() const;
   
-  inline arma_warn_unused bool is_symmetric() const;
-  inline arma_warn_unused bool is_symmetric(const typename get_pod_type<elem_type>::result tol) const;
+  arma_warn_unused inline bool is_symmetric() const;
+  arma_warn_unused inline bool is_symmetric(const typename get_pod_type<elem_type>::result tol) const;
   
-  inline arma_warn_unused bool is_hermitian() const;
-  inline arma_warn_unused bool is_hermitian(const typename get_pod_type<elem_type>::result tol) const;
+  arma_warn_unused inline bool is_hermitian() const;
+  arma_warn_unused inline bool is_hermitian(const typename get_pod_type<elem_type>::result tol) const;
   
-  inline arma_warn_unused bool is_zero(const typename get_pod_type<elem_type>::result tol = 0) const;
+  arma_warn_unused inline bool is_zero(const typename get_pod_type<elem_type>::result tol = 0) const;
   
-  inline arma_warn_unused bool is_trimatu() const;
-  inline arma_warn_unused bool is_trimatl() const;
-  inline arma_warn_unused bool is_diagmat() const;
-  inline arma_warn_unused bool is_empty()   const;
-  inline arma_warn_unused bool is_square()  const;
-  inline arma_warn_unused bool is_vec()     const;
-  inline arma_warn_unused bool is_colvec()  const;
-  inline arma_warn_unused bool is_rowvec()  const;
-  inline arma_warn_unused bool is_finite()  const;
-  inline arma_warn_unused bool has_inf()    const;
-  inline arma_warn_unused bool has_nan()    const;
+  arma_warn_unused inline bool is_trimatu() const;
+  arma_warn_unused inline bool is_trimatl() const;
+  arma_warn_unused inline bool is_diagmat() const;
+  arma_warn_unused inline bool is_empty()   const;
+  arma_warn_unused inline bool is_square()  const;
+  arma_warn_unused inline bool is_vec()     const;
+  arma_warn_unused inline bool is_colvec()  const;
+  arma_warn_unused inline bool is_rowvec()  const;
+  arma_warn_unused inline bool is_finite()  const;
   
-  inline arma_warn_unused const Op<derived,op_vectorise_col> as_col() const;
-  inline arma_warn_unused const Op<derived,op_vectorise_row> as_row() const;
+  arma_warn_unused inline bool has_inf()       const;
+  arma_warn_unused inline bool has_nan()       const;
+  arma_warn_unused inline bool has_nonfinite() const;
+  
+  arma_warn_unused inline const Op<derived,op_vectorise_col> as_col() const;
+  arma_warn_unused inline const Op<derived,op_vectorise_row> as_row() const;
   };
 
 
