@@ -56,9 +56,9 @@ class SpRow : public SpMat<eT>
   template<typename T1, typename T2>
   inline explicit SpRow(const SpBase<pod_type,T1>& A, const SpBase<pod_type,T2>& B);
   
-  inline arma_warn_unused const SpOp<SpRow<eT>,spop_htrans>  t() const;
-  inline arma_warn_unused const SpOp<SpRow<eT>,spop_htrans> ht() const;
-  inline arma_warn_unused const SpOp<SpRow<eT>,spop_strans> st() const;
+  arma_warn_unused inline const SpOp<SpRow<eT>,spop_htrans>  t() const;
+  arma_warn_unused inline const SpOp<SpRow<eT>,spop_htrans> ht() const;
+  arma_warn_unused inline const SpOp<SpRow<eT>,spop_strans> st() const;
   
   inline void shed_col (const uword col_num);
   inline void shed_cols(const uword in_col1, const uword in_col2);
@@ -75,7 +75,7 @@ class SpRow : public SpMat<eT>
   inline       row_iterator end_row(const uword row_num = 0);
   inline const_row_iterator end_row(const uword row_num = 0) const;
   
-  #ifdef ARMA_EXTRA_SPROW_PROTO
+  #if defined(ARMA_EXTRA_SPROW_PROTO)
     #include ARMA_INCFILE_WRAP(ARMA_EXTRA_SPROW_PROTO)
   #endif
   };
