@@ -104,24 +104,24 @@ class SpSubview : public SpBase< eT, SpSubview<eT> >
   inline void randn();
   
   
-  arma_hot inline SpSubview_MapMat_val<eT> operator[](const uword i);
-  arma_hot inline eT                       operator[](const uword i) const;
+  arma_warn_unused inline SpSubview_MapMat_val<eT> operator[](const uword i);
+  arma_warn_unused inline eT                       operator[](const uword i) const;
   
-  arma_hot inline SpSubview_MapMat_val<eT> operator()(const uword i);
-  arma_hot inline eT                       operator()(const uword i) const;
+  arma_warn_unused inline SpSubview_MapMat_val<eT> operator()(const uword i);
+  arma_warn_unused inline eT                       operator()(const uword i) const;
   
-  arma_hot inline SpSubview_MapMat_val<eT> operator()(const uword in_row, const uword in_col);
-  arma_hot inline eT                       operator()(const uword in_row, const uword in_col) const;
+  arma_warn_unused inline SpSubview_MapMat_val<eT> operator()(const uword in_row, const uword in_col);
+  arma_warn_unused inline eT                       operator()(const uword in_row, const uword in_col) const;
   
-  arma_hot inline SpSubview_MapMat_val<eT> at(const uword i);
-  arma_hot inline eT                       at(const uword i) const;
+  arma_warn_unused inline SpSubview_MapMat_val<eT> at(const uword i);
+  arma_warn_unused inline eT                       at(const uword i) const;
   
-  arma_hot inline SpSubview_MapMat_val<eT> at(const uword in_row, const uword in_col);
-  arma_hot inline eT                       at(const uword in_row, const uword in_col) const;
+  arma_warn_unused inline SpSubview_MapMat_val<eT> at(const uword in_row, const uword in_col);
+  arma_warn_unused inline eT                       at(const uword in_row, const uword in_col) const;
   
   inline bool check_overlap(const SpSubview& x) const;
   
-  inline bool is_vec() const;
+  arma_warn_unused inline bool is_vec() const;
   
   inline       SpSubview_row<eT> row(const uword row_num);
   inline const SpSubview_row<eT> row(const uword row_num) const;
@@ -197,23 +197,23 @@ class SpSubview : public SpBase< eT, SpSubview<eT> >
     // Don't hold location internally; call "dummy" methods to get that information.
     arma_inline uword row() const { return iterator_base::M->m.row_indices[iterator_base::internal_pos + skip_pos] - iterator_base::M->aux_row1; }
     
-    inline arma_hot         const_iterator& operator++();
-    inline arma_warn_unused const_iterator  operator++(int);
+    arma_hot         inline const_iterator& operator++();
+    arma_warn_unused inline const_iterator  operator++(int);
     
-    inline arma_hot         const_iterator& operator--();
-    inline arma_warn_unused const_iterator  operator--(int);
+    arma_hot         inline const_iterator& operator--();
+    arma_warn_unused inline const_iterator  operator--(int);
     
-    inline arma_hot bool operator!=(const const_iterator& rhs) const;
-    inline arma_hot bool operator==(const const_iterator& rhs) const;
+    arma_hot inline bool operator!=(const const_iterator& rhs) const;
+    arma_hot inline bool operator==(const const_iterator& rhs) const;
     
-    inline arma_hot bool operator!=(const typename SpMat<eT>::const_iterator& rhs) const;
-    inline arma_hot bool operator==(const typename SpMat<eT>::const_iterator& rhs) const;
+    arma_hot inline bool operator!=(const typename SpMat<eT>::const_iterator& rhs) const;
+    arma_hot inline bool operator==(const typename SpMat<eT>::const_iterator& rhs) const;
     
-    inline arma_hot bool operator!=(const const_row_iterator& rhs) const;
-    inline arma_hot bool operator==(const const_row_iterator& rhs) const;
+    arma_hot inline bool operator!=(const const_row_iterator& rhs) const;
+    arma_hot inline bool operator==(const const_row_iterator& rhs) const;
     
-    inline arma_hot bool operator!=(const typename SpMat<eT>::const_row_iterator& rhs) const;
-    inline arma_hot bool operator==(const typename SpMat<eT>::const_row_iterator& rhs) const;
+    arma_hot inline bool operator!=(const typename SpMat<eT>::const_row_iterator& rhs) const;
+    arma_hot inline bool operator==(const typename SpMat<eT>::const_row_iterator& rhs) const;
     
     arma_aligned uword skip_pos; // not used in row_iterator or const_row_iterator
     };
@@ -227,14 +227,14 @@ class SpSubview : public SpBase< eT, SpSubview<eT> >
     inline iterator(SpSubview& in_M, const uword in_row, const uword in_col, const uword in_pos, const uword in_skip_pos) : const_iterator(in_M, in_row, in_col, in_pos, in_skip_pos) { }
     inline iterator(const iterator& other) : const_iterator(other) { }
     
-    inline arma_hot SpValProxy< SpSubview<eT> > operator*();
+    arma_hot inline SpValProxy< SpSubview<eT> > operator*();
     
     // overloads needed for return type correctness
-    inline arma_hot         iterator& operator++();
-    inline arma_warn_unused iterator  operator++(int);
+    arma_hot         inline iterator& operator++();
+    arma_warn_unused inline iterator  operator++(int);
     
-    inline arma_hot         iterator& operator--();
-    inline arma_warn_unused iterator  operator--(int);
+    arma_hot         inline iterator& operator--();
+    arma_warn_unused inline iterator  operator--(int);
     
     // This has a different value_type than iterator_base.
     typedef       SpValProxy< SpSubview<eT> >  value_type;
@@ -251,11 +251,11 @@ class SpSubview : public SpBase< eT, SpSubview<eT> >
     inline const_row_iterator(const SpSubview& in_M, uword in_row, uword in_col);
     inline const_row_iterator(const const_row_iterator& other);
     
-    inline arma_hot         const_row_iterator& operator++();
-    inline arma_warn_unused const_row_iterator  operator++(int);
+    arma_hot         inline const_row_iterator& operator++();
+    arma_warn_unused inline const_row_iterator  operator++(int);
     
-    inline arma_hot         const_row_iterator& operator--();
-    inline arma_warn_unused const_row_iterator  operator--(int);
+    arma_hot         inline const_row_iterator& operator--();
+    arma_warn_unused inline const_row_iterator  operator--(int);
     
     uword internal_row; // Hold row internally because we use internal_pos differently.
     uword actual_pos; // Actual position in subview's parent matrix.
@@ -264,17 +264,17 @@ class SpSubview : public SpBase< eT, SpSubview<eT> >
     
     arma_inline uword row() const { return internal_row; }
     
-    inline arma_hot bool operator!=(const const_iterator& rhs) const;
-    inline arma_hot bool operator==(const const_iterator& rhs) const;
+    arma_hot inline bool operator!=(const const_iterator& rhs) const;
+    arma_hot inline bool operator==(const const_iterator& rhs) const;
     
-    inline arma_hot bool operator!=(const typename SpMat<eT>::const_iterator& rhs) const;
-    inline arma_hot bool operator==(const typename SpMat<eT>::const_iterator& rhs) const;
+    arma_hot inline bool operator!=(const typename SpMat<eT>::const_iterator& rhs) const;
+    arma_hot inline bool operator==(const typename SpMat<eT>::const_iterator& rhs) const;
     
-    inline arma_hot bool operator!=(const const_row_iterator& rhs) const;
-    inline arma_hot bool operator==(const const_row_iterator& rhs) const;
+    arma_hot inline bool operator!=(const const_row_iterator& rhs) const;
+    arma_hot inline bool operator==(const const_row_iterator& rhs) const;
     
-    inline arma_hot bool operator!=(const typename SpMat<eT>::const_row_iterator& rhs) const;
-    inline arma_hot bool operator==(const typename SpMat<eT>::const_row_iterator& rhs) const;
+    arma_hot inline bool operator!=(const typename SpMat<eT>::const_row_iterator& rhs) const;
+    arma_hot inline bool operator==(const typename SpMat<eT>::const_row_iterator& rhs) const;
     };
   
   class row_iterator : public const_row_iterator
@@ -285,14 +285,14 @@ class SpSubview : public SpBase< eT, SpSubview<eT> >
     inline row_iterator(SpSubview& in_M, uword in_row, uword in_col) : const_row_iterator(in_M, in_row, in_col) { }
     inline row_iterator(const row_iterator& other) : const_row_iterator(other) { }
     
-    inline arma_hot SpValProxy< SpSubview<eT> > operator*();
+    arma_hot inline SpValProxy< SpSubview<eT> > operator*();
     
     // overloads needed for return type correctness
-    inline arma_hot         row_iterator& operator++();
-    inline arma_warn_unused row_iterator  operator++(int);
+    arma_hot         inline row_iterator& operator++();
+    arma_warn_unused inline row_iterator  operator++(int);
     
-    inline arma_hot         row_iterator& operator--();
-    inline arma_warn_unused row_iterator  operator--(int);
+    arma_hot         inline row_iterator& operator--();
+    arma_warn_unused inline row_iterator  operator--(int);
     
     // This has a different value_type than iterator_base.
     typedef       SpValProxy< SpSubview<eT> >  value_type;
@@ -331,8 +331,8 @@ class SpSubview : public SpBase< eT, SpSubview<eT> >
   friend class SpSubview_row<eT>;
   friend class SpValProxy< SpSubview<eT> >;   // allow SpValProxy to call insert_element() and delete_element()
   
-  inline arma_warn_unused eT&  insert_element(const uword in_row, const uword in_col, const eT in_val = eT(0));
-  inline                  void delete_element(const uword in_row, const uword in_col);
+  arma_warn_unused inline eT&  insert_element(const uword in_row, const uword in_col, const eT in_val = eT(0));
+                   inline void delete_element(const uword in_row, const uword in_col);
   
   inline void invalidate_cache() const;
   };
@@ -357,9 +357,9 @@ class SpSubview_col : public SpSubview<eT>
   template<typename T1> inline void operator= (const SpBase<eT,T1>& x);
   template<typename T1> inline void operator= (const   Base<eT,T1>& x);
   
-  inline arma_warn_unused const SpOp<SpSubview_col<eT>,spop_htrans>  t() const;
-  inline arma_warn_unused const SpOp<SpSubview_col<eT>,spop_htrans> ht() const;
-  inline arma_warn_unused const SpOp<SpSubview_col<eT>,spop_strans> st() const;
+  arma_warn_unused inline const SpOp<SpSubview_col<eT>,spop_htrans>  t() const;
+  arma_warn_unused inline const SpOp<SpSubview_col<eT>,spop_htrans> ht() const;
+  arma_warn_unused inline const SpOp<SpSubview_col<eT>,spop_strans> st() const;
   
   
   protected:
@@ -395,9 +395,9 @@ class SpSubview_row : public SpSubview<eT>
   template<typename T1> inline void operator= (const SpBase<eT,T1>& x);
   template<typename T1> inline void operator= (const   Base<eT,T1>& x);
   
-  inline arma_warn_unused const SpOp<SpSubview_row<eT>,spop_htrans>  t() const;
-  inline arma_warn_unused const SpOp<SpSubview_row<eT>,spop_htrans> ht() const;
-  inline arma_warn_unused const SpOp<SpSubview_row<eT>,spop_strans> st() const;
+  arma_warn_unused inline const SpOp<SpSubview_row<eT>,spop_htrans>  t() const;
+  arma_warn_unused inline const SpOp<SpSubview_row<eT>,spop_htrans> ht() const;
+  arma_warn_unused inline const SpOp<SpSubview_row<eT>,spop_strans> st() const;
   
   
   protected:
