@@ -20,55 +20,63 @@
 //! @{
 
 
-#if defined(ARMA_RNG_ALT)
-  #undef ARMA_USE_EXTERN_RNG
+#undef  ARMA_USE_CXX11_RNG
+#define ARMA_USE_CXX11_RNG
+
+#undef  ARMA_USE_THREAD_LOCAL
+#define ARMA_USE_THREAD_LOCAL
+
+#if (defined(ARMA_RNG_ALT) || defined(ARMA_DONT_USE_CXX11_RNG))
+  #undef ARMA_USE_CXX11_RNG
+#endif
+
+#if defined(ARMA_DONT_USE_THREAD_LOCAL)
+  #undef ARMA_USE_THREAD_LOCAL
 #endif
 
 
-// NOTE: mt19937_64_instance_warmup is used as a workaround
+// NOTE: ARMA_WARMUP_PRODUCER enables a workaround 
 // NOTE: for thread_local issue on macOS 11 and/or AppleClang 12.0
 // NOTE: see https://gitlab.com/conradsnicta/armadillo-code/-/issues/173
 // NOTE: if this workaround causes problems, please report it and 
-// NOTE: disable the workaround by uncommenting the code block below:
+// NOTE: disable the workaround by commenting out the code block below:
 
-// #if defined(__APPLE__) || defined(__apple_build_version__)
-//   #if !defined(ARMA_DONT_DISABLE_EXTERN_RNG)
-//     #undef ARMA_USE_EXTERN_RNG
-//   #endif
-// #endif
+#if defined(__APPLE__) || defined(__apple_build_version__)
+  #undef  ARMA_WARMUP_PRODUCER
+  #define ARMA_WARMUP_PRODUCER
+#endif
 
+#if defined(ARMA_DONT_WARMUP_PRODUCER)
+  #undef ARMA_WARMUP_PRODUCER
+#endif
 
 // NOTE: workaround for another thread_local issue on macOS
 // NOTE: where GCC (not Clang) may not have support for thread_local
 
 #if (defined(__APPLE__) && defined(__GNUG__) && !defined(__clang__))
-  #if !defined(ARMA_DONT_DISABLE_EXTERN_RNG)
-    #undef ARMA_USE_EXTERN_RNG
-  #endif
+  #undef ARMA_USE_THREAD_LOCAL
 #endif
 
+// NOTE: disable use of thread_local on MinGW et al;
+// NOTE: i don't have the patience to keep looking into these broken platforms
 
-
-#if defined(ARMA_USE_EXTERN_RNG)
-  extern thread_local std::mt19937_64 mt19937_64_instance;
-  
-  #if defined(__APPLE__) || defined(__apple_build_version__)
-    namespace
-      {
-      struct mt19937_64_instance_warmup
-        {
-        inline mt19937_64_instance_warmup()
-          {
-          typename std::mt19937_64::result_type junk = mt19937_64_instance();
-          arma_ignore(junk);
-          }
-        };
-      
-      static mt19937_64_instance_warmup mt19937_64_instance_warmup_run;
-      }
-  #endif
+#if (defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__) || defined(__MSYS__) || defined(__MSYS2__))
+  #undef ARMA_USE_THREAD_LOCAL
 #endif
 
+#if defined(ARMA_FORCE_USE_THREAD_LOCAL)
+  #undef  ARMA_USE_THREAD_LOCAL
+  #define ARMA_USE_THREAD_LOCAL
+#endif
+
+#if (!defined(ARMA_USE_THREAD_LOCAL))
+  #undef  ARMA_GUARD_PRODUCER
+  #define ARMA_GUARD_PRODUCER
+#endif
+
+#if (defined(ARMA_DONT_GUARD_PRODUCER) || defined(ARMA_DONT_USE_STD_MUTEX))
+  #undef ARMA_GUARD_PRODUCER
+#endif
 
 
 class arma_rng
@@ -77,20 +85,32 @@ class arma_rng
   
   #if   defined(ARMA_RNG_ALT)
     typedef arma_rng_alt::seed_type      seed_type;
-  #elif defined(ARMA_USE_EXTERN_RNG)
+  #elif defined(ARMA_USE_CXX11_RNG)
     typedef std::mt19937_64::result_type seed_type;
   #else
-    typedef arma_rng_cxx98::seed_type    seed_type;
+    typedef arma_rng_cxx03::seed_type    seed_type;
   #endif
   
   #if   defined(ARMA_RNG_ALT)
     static constexpr int rng_method = 2;
-  #elif defined(ARMA_USE_EXTERN_RNG)
+  #elif defined(ARMA_USE_CXX11_RNG)
     static constexpr int rng_method = 1;
   #else
     static constexpr int rng_method = 0;
   #endif
   
+  #if defined(ARMA_USE_CXX11_RNG)
+    inline static std::mt19937_64&    get_producer();
+    inline static void             warmup_producer(std::mt19937_64& producer);
+    
+    inline static void   lock_producer();
+    inline static void unlock_producer();
+    
+    #if defined(ARMA_GUARD_PRODUCER)
+      inline static std::mutex& get_producer_mutex();
+    #endif
+  #endif
+  
   inline static void set_seed(const seed_type val);
   inline static void set_seed_random();
   
@@ -102,6 +122,101 @@ class arma_rng
 
 
 
+#if defined(ARMA_USE_CXX11_RNG)
+
+inline
+std::mt19937_64&
+arma_rng::get_producer()
+  {
+  #if defined(ARMA_USE_THREAD_LOCAL)
+    
+    // use a thread-safe RNG, with each thread having its own unique starting seed
+    
+    static std::atomic<std::size_t> mt19937_64_producer_counter(0);
+    
+    static thread_local std::mt19937_64 mt19937_64_producer( std::mt19937_64::default_seed + mt19937_64_producer_counter++ );
+    
+    arma_rng::warmup_producer(mt19937_64_producer);
+    
+  #else
+    
+    // use a plain RNG in case we don't have thread_local
+    
+    static std::mt19937_64 mt19937_64_producer( std::mt19937_64::default_seed );
+    
+    arma_rng::warmup_producer(mt19937_64_producer);
+    
+  #endif
+  
+  return mt19937_64_producer;
+  }
+
+
+inline
+void
+arma_rng::warmup_producer(std::mt19937_64& producer)
+  {
+  #if defined(ARMA_WARMUP_PRODUCER)
+    
+    static std::atomic_flag warmup_done = ATOMIC_FLAG_INIT;  // init to false
+    
+    if(warmup_done.test_and_set() == false)
+      {
+      typename std::mt19937_64::result_type junk = producer();
+      
+      arma_ignore(junk);
+      }
+    
+  #else
+    
+    arma_ignore(producer);
+    
+  #endif
+  }
+
+
+inline
+void
+arma_rng::lock_producer()
+  {
+  #if defined(ARMA_GUARD_PRODUCER)
+    
+    std::mutex& producer_mutex = arma_rng::get_producer_mutex();
+    
+    producer_mutex.lock();
+    
+  #endif
+  }
+
+
+inline
+void
+arma_rng::unlock_producer()
+  {
+  #if defined(ARMA_GUARD_PRODUCER)
+    
+    std::mutex& producer_mutex = arma_rng::get_producer_mutex();
+    
+    producer_mutex.unlock();
+    
+  #endif
+  }
+
+
+#if defined(ARMA_GUARD_PRODUCER)
+  inline
+  std::mutex&
+  arma_rng::get_producer_mutex()
+    {
+    static std::mutex producer_mutex;
+    
+    return producer_mutex;
+    }
+#endif
+
+#endif
+
+
 inline
 void
 arma_rng::set_seed(const arma_rng::seed_type val)
@@ -110,13 +225,15 @@ arma_rng::set_seed(const arma_rng::seed_type val)
     {
     arma_rng_alt::set_seed(val);
     }
-  #elif defined(ARMA_USE_EXTERN_RNG)
+  #elif defined(ARMA_USE_CXX11_RNG)
     {
-    mt19937_64_instance.seed(val);
+    arma_rng::lock_producer();
+    arma_rng::get_producer().seed(val);
+    arma_rng::unlock_producer();
     }
   #else
     {
-    arma_rng_cxx98::set_seed(val);
+    arma_rng_cxx03::set_seed(val);
     }
   #endif
   }
@@ -141,7 +258,7 @@ arma_rng::set_seed_random()
     
     if(rd.entropy() > double(0))  { seed1 = static_cast<seed_type>( rd() ); }
     
-    if(seed1 != seed_type(0))  { have_seed = true; }
+    have_seed = (seed1 != seed_type(0));
     }
   catch(...) {}
   
@@ -162,12 +279,9 @@ arma_rng::set_seed_random()
       
       if(f.good())  { f.read((char*)(&(tmp.b[0])), sizeof(seed_type)); }
       
-      if(f.good())
-        {
-        seed2 = tmp.a;
+      if(f.good())  { seed2 = tmp.a; }
         
-        if(seed2 != seed_type(0))  { have_seed = true; }
-        }
+      have_seed = (seed2 != seed_type(0));
       }
     catch(...) {}
     }
@@ -199,7 +313,7 @@ arma_rng::set_seed_random()
       }
     }
   
-  arma_rng::set_seed( seed1 + seed2 + seed3 + seed4 );
+  arma_rng::set_seed(seed1 + seed2 + seed3 + seed4);
   }
 
 
@@ -218,15 +332,21 @@ struct arma_rng::randi
       {
       return eT( arma_rng_alt::randi_val() );
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       constexpr double scale = double(std::numeric_limits<int>::max()) / double(std::mt19937_64::max());
       
-      return eT( double(mt19937_64_instance()) * scale );
+      arma_rng::lock_producer();
+      
+      const eT out = eT(double(arma_rng::get_producer()()) * scale);
+      
+      arma_rng::unlock_producer();
+      
+      return out;
       }
     #else
       {
-      return eT( arma_rng_cxx98::randi_val() );
+      return eT( arma_rng_cxx03::randi_val() );
       }
     #endif
     }
@@ -241,13 +361,13 @@ struct arma_rng::randi
       {
       return arma_rng_alt::randi_max_val();
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       return std::numeric_limits<int>::max();
       }
     #else
       {
-      return arma_rng_cxx98::randi_max_val();
+      return arma_rng_cxx03::randi_max_val();
       }
     #endif
     }
@@ -262,15 +382,21 @@ struct arma_rng::randi
       {
       arma_rng_alt::randi_fill(mem, N, a, b);
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::uniform_int_distribution<int> local_i_distr(a, b);
       
-      for(uword i=0; i<N; ++i)  { mem[i] = eT(local_i_distr(mt19937_64_instance)); }
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      for(uword i=0; i<N; ++i)  { mem[i] = eT(local_i_distr(producer)); }
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
-      if(N == uword(1))  { arma_rng_cxx98::randi_fill(mem, uword(1), a, b); return; }
+      if(N == uword(1))  { arma_rng_cxx03::randi_fill(mem, uword(1), a, b); return; }
       
       typedef typename std::mt19937_64::result_type local_seed_type;
       
@@ -301,15 +427,21 @@ struct arma_rng::randu
       {
       return eT( arma_rng_alt::randu_val() );
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       constexpr double scale = double(1.0) / double(std::mt19937_64::max());
       
-      return eT( double(mt19937_64_instance()) * scale );
+      arma_rng::lock_producer();
+      
+      const eT out = eT( double(arma_rng::get_producer()()) * scale );
+      
+      arma_rng::unlock_producer();
+      
+      return out;
       }
     #else
       {
-      return eT( arma_rng_cxx98::randu_val() );
+      return eT( arma_rng_cxx03::randu_val() );
       }
     #endif
     }
@@ -324,15 +456,21 @@ struct arma_rng::randu
       {
       for(uword i=0; i < N; ++i)  { mem[i] = eT( arma_rng_alt::randu_val() ); }
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::uniform_real_distribution<double> local_u_distr;
       
-      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_u_distr(mt19937_64_instance) ); }
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_u_distr(producer) ); }
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
-      if(N == uword(1))  { mem[0] = eT( arma_rng_cxx98::randu_val() ); return; }
+      if(N == uword(1))  { mem[0] = eT( arma_rng_cxx03::randu_val() ); return; }
       
       typedef typename std::mt19937_64::result_type local_seed_type;
       
@@ -345,6 +483,46 @@ struct arma_rng::randu
       }
     #endif
     }
+  
+  
+  inline
+  static
+  void
+  fill(eT* mem, const uword N, const double a, const double b)
+    {
+    #if defined(ARMA_RNG_ALT)
+      {
+      const double r = b - a;
+      
+      for(uword i=0; i < N; ++i)  { mem[i] = eT( arma_rng_alt::randu_val() * r + a ); }
+      }
+    #elif defined(ARMA_USE_CXX11_RNG)
+      {
+      std::uniform_real_distribution<double> local_u_distr(a,b);
+      
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_u_distr(producer) ); }
+      
+      arma_rng::unlock_producer();
+      }
+    #else
+      {
+      if(N == uword(1))  { mem[0] = eT( arma_rng_cxx03::randu_val() * (b - a) + a ); return; }
+      
+      typedef typename std::mt19937_64::result_type local_seed_type;
+      
+      std::mt19937_64                        local_engine;
+      std::uniform_real_distribution<double> local_u_distr(a,b);
+      
+      local_engine.seed( local_seed_type(std::rand()) );
+      
+      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_u_distr(local_engine) ); }
+      }
+    #endif
+    }
   };
 
 
@@ -362,19 +540,25 @@ struct arma_rng::randu< std::complex<T> >
       
       return std::complex<T>(a, b);
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::uniform_real_distribution<double> local_u_distr;
       
-      const T a = T( local_u_distr(mt19937_64_instance) );
-      const T b = T( local_u_distr(mt19937_64_instance) );
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      const T a = T( local_u_distr(producer) );
+      const T b = T( local_u_distr(producer) );
+      
+      arma_rng::unlock_producer();
       
       return std::complex<T>(a, b);
       }
     #else
       {
-      const T a = T( arma_rng_cxx98::randu_val() );
-      const T b = T( arma_rng_cxx98::randu_val() );
+      const T a = T( arma_rng_cxx03::randu_val() );
+      const T b = T( arma_rng_cxx03::randu_val() );
       
       return std::complex<T>(a, b);
       }
@@ -397,24 +581,30 @@ struct arma_rng::randu< std::complex<T> >
         mem[i] = std::complex<T>(a, b);
         }
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::uniform_real_distribution<double> local_u_distr;
       
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
       for(uword i=0; i < N; ++i)
         {
-        const T a = T( local_u_distr(mt19937_64_instance) );
-        const T b = T( local_u_distr(mt19937_64_instance) );
+        const T a = T( local_u_distr(producer) );
+        const T b = T( local_u_distr(producer) );
         
         mem[i] = std::complex<T>(a, b);
         }
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
       if(N == uword(1))
         {
-        const T a = T( arma_rng_cxx98::randu_val() );
-        const T b = T( arma_rng_cxx98::randu_val() );
+        const T a = T( arma_rng_cxx03::randu_val() );
+        const T b = T( arma_rng_cxx03::randu_val() );
         
         mem[0] = std::complex<T>(a, b);
         
@@ -438,6 +628,74 @@ struct arma_rng::randu< std::complex<T> >
       }
     #endif
     }
+  
+  
+  inline
+  static
+  void
+  fill(std::complex<T>* mem, const uword N, const double a, const double b)
+    {
+    #if defined(ARMA_RNG_ALT)
+      {
+      const double r = b - a;
+      
+      for(uword i=0; i < N; ++i)
+        {
+        const T tmp1 = T( arma_rng_alt::randu_val() * r + a );
+        const T tmp2 = T( arma_rng_alt::randu_val() * r + a );
+        
+        mem[i] = std::complex<T>(tmp1, tmp2);
+        }
+      }
+    #elif defined(ARMA_USE_CXX11_RNG)
+      {
+      std::uniform_real_distribution<double> local_u_distr(a,b);
+      
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      for(uword i=0; i < N; ++i)
+        {
+        const T tmp1 = T( local_u_distr(producer) );
+        const T tmp2 = T( local_u_distr(producer) );
+        
+        mem[i] = std::complex<T>(tmp1, tmp2);
+        }
+      
+      arma_rng::unlock_producer();
+      }
+    #else
+      {
+      if(N == uword(1))
+        {
+        const double r = b - a;
+        
+        const T tmp1 = T( arma_rng_cxx03::randu_val() * r + a);
+        const T tmp2 = T( arma_rng_cxx03::randu_val() * r + a);
+        
+        mem[0] = std::complex<T>(tmp1, tmp2);
+        
+        return;
+        }
+      
+      typedef typename std::mt19937_64::result_type local_seed_type;
+      
+      std::mt19937_64                        local_engine;
+      std::uniform_real_distribution<double> local_u_distr(a,b);
+      
+      local_engine.seed( local_seed_type(std::rand()) );
+      
+      for(uword i=0; i < N; ++i)
+        {
+        const T tmp1 = T( local_u_distr(local_engine) );
+        const T tmp2 = T( local_u_distr(local_engine) );
+        
+        mem[i] = std::complex<T>(tmp1, tmp2);
+        }
+      }
+    #endif
+    }
   };
 
 
@@ -456,15 +714,21 @@ struct arma_rng::randn
       {
       return eT( arma_rng_alt::randn_val() );
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::normal_distribution<double> local_n_distr;
       
-      return eT( local_n_distr(mt19937_64_instance) );
+      arma_rng::lock_producer();
+      
+      const eT out = eT( local_n_distr(arma_rng::get_producer()) );
+      
+      arma_rng::unlock_producer();
+      
+      return out;
       }
     #else
       {
-      return eT( arma_rng_cxx98::randn_val() );
+      return eT( arma_rng_cxx03::randn_val() );
       }
     #endif
     }
@@ -479,16 +743,22 @@ struct arma_rng::randn
       {
       arma_rng_alt::randn_dual_val(out1, out2);
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::normal_distribution<double> local_n_distr;
       
-      out1 = eT( local_n_distr(mt19937_64_instance) );
-      out2 = eT( local_n_distr(mt19937_64_instance) );
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      out1 = eT( local_n_distr(producer) );
+      out2 = eT( local_n_distr(producer) );
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
-      arma_rng_cxx98::randn_dual_val(out1, out2);
+      arma_rng_cxx03::randn_dual_val(out1, out2);
       }
     #endif
     }
@@ -497,7 +767,7 @@ struct arma_rng::randn
   inline
   static
   void
-  fill_simple(eT* mem, const uword N)
+  fill(eT* mem, const uword N)
     {
     #if defined(ARMA_RNG_ALT)
       {
@@ -509,15 +779,21 @@ struct arma_rng::randn
       
       if(i < N)  { mem[i] = eT( arma_rng_alt::randn_val() ); }
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::normal_distribution<double> local_n_distr;
       
-      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_n_distr(mt19937_64_instance) ); }
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_n_distr(producer) ); }
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
-      if(N == uword(1))  { mem[0] = eT( arma_rng_cxx98::randn_val() ); return; }
+      if(N == uword(1))  { mem[0] = eT( arma_rng_cxx03::randn_val() ); return; }
       
       typedef typename std::mt19937_64::result_type local_seed_type;
       
@@ -535,52 +811,66 @@ struct arma_rng::randn
   inline
   static
   void
-  fill(eT* mem, const uword N)
+  fill(eT* mem, const uword N, const double mu, const double sd)
     {
-    #if defined(ARMA_USE_OPENMP)
+    #if defined(ARMA_RNG_ALT)
       {
-      if((N < 1024) || omp_in_parallel())  { arma_rng::randn<eT>::fill_simple(mem, N); return; }
+      // NOTE: old method to avoid regressions in user code that assumes specific sequence
       
-      typedef typename std::mt19937_64::result_type local_seed_type;
+      uword i, j;
       
-      const uword n_threads = uword( mp_thread_limit::get() );
-      
-      std::vector< std::mt19937_64                  > engine(n_threads);
-      std::vector< std::normal_distribution<double> >  distr(n_threads);
-      
-      for(uword t=0; t < n_threads; ++t)
+      for(i=0, j=1; j < N; i+=2, j+=2)
         {
-        std::mt19937_64& t_engine = engine[t];
+        eT val_i = eT(0);
+        eT val_j = eT(0);
         
-        t_engine.seed( local_seed_type(t) + local_seed_type(arma_rng::randi<local_seed_type>()) );
+        arma_rng_alt::randn_dual_val( val_i, val_j );
+        
+        mem[i] = (val_i * sd) + mu;
+        mem[j] = (val_j * sd) + mu;
         }
       
-      const uword chunk_size = N / n_threads;
-      
-      #pragma omp parallel for schedule(static) num_threads(int(n_threads))
-      for(uword t=0; t < n_threads; ++t)
+      if(i < N)
         {
-        const uword start = (t+0) * chunk_size;
-        const uword endp1 = (t+1) * chunk_size;
-        
-        std::mt19937_64&                  t_engine = engine[t];
-        std::normal_distribution<double>& t_distr  =  distr[t];
-        
-        for(uword i=start; i < endp1; ++i)  { mem[i] = eT( t_distr(t_engine)); }
+        const eT val_i = eT( arma_rng_alt::randn_val() );
+         
+        mem[i] = (val_i * sd) + mu;
         }
+      }
+    #elif defined(ARMA_USE_CXX11_RNG)
+      {
+      std::normal_distribution<double> local_n_distr(mu, sd);
       
-      std::mt19937_64&                  t0_engine = engine[0];
-      std::normal_distribution<double>& t0_distr  =  distr[0];
+      std::mt19937_64& producer = arma_rng::get_producer();
       
-      for(uword i=(n_threads*chunk_size); i < N; ++i)  { mem[i] = eT( t0_distr(t0_engine)); }
+      arma_rng::lock_producer();
+      
+      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_n_distr(producer) ); }
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
-      arma_rng::randn<eT>::fill_simple(mem, N);
+      if(N == uword(1))
+        {
+        const eT val = eT( arma_rng_cxx03::randn_val() );
+        
+        mem[0] = (val * sd) + mu;
+        
+        return;
+        }
+      
+      typedef typename std::mt19937_64::result_type local_seed_type;
+      
+      std::mt19937_64                  local_engine;
+      std::normal_distribution<double> local_n_distr(mu, sd);
+      
+      local_engine.seed( local_seed_type(std::rand()) );
+      
+      for(uword i=0; i < N; ++i)  { mem[i] = eT( local_n_distr(local_engine) ); }
       }
     #endif
     }
-  
   };
 
 
@@ -631,23 +921,29 @@ struct arma_rng::randn< std::complex<T> >
   inline
   static
   void
-  fill_simple(std::complex<T>* mem, const uword N)
+  fill(std::complex<T>* mem, const uword N)
     {
     #if defined(ARMA_RNG_ALT)
       {
       for(uword i=0; i < N; ++i)  { mem[i] = std::complex<T>( arma_rng::randn< std::complex<T> >() ); }
       }
-    #elif defined(ARMA_USE_EXTERN_RNG)
+    #elif defined(ARMA_USE_CXX11_RNG)
       {
       std::normal_distribution<double> local_n_distr;
       
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
       for(uword i=0; i < N; ++i)
         {
-        const T a = T( local_n_distr(mt19937_64_instance) );
-        const T b = T( local_n_distr(mt19937_64_instance) );
+        const T a = T( local_n_distr(producer) );
+        const T b = T( local_n_distr(producer) );
         
         mem[i] = std::complex<T>(a,b);
         }
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
@@ -656,7 +952,7 @@ struct arma_rng::randn< std::complex<T> >
         T a = T(0);
         T b = T(0);
         
-        arma_rng_cxx98::randn_dual_val(a,b);
+        arma_rng_cxx03::randn_dual_val(a,b);
         
         mem[0] = std::complex<T>(a,b);
         
@@ -685,62 +981,18 @@ struct arma_rng::randn< std::complex<T> >
   inline
   static
   void
-  fill(std::complex<T>* mem, const uword N)
+  fill(std::complex<T>* mem, const uword N, const double mu, const double sd)
     {
-    #if defined(ARMA_USE_OPENMP)
+    arma_rng::randn< std::complex<T> >::fill(mem, N);
+    
+    if( (mu == double(0)) && (sd == double(1)) )  { return; }
+    
+    for(uword i=0; i<N; ++i)
       {
-      if((N < 512) || omp_in_parallel())  { arma_rng::randn< std::complex<T> >::fill_simple(mem, N); return; }
+      const std::complex<T>& val = mem[i];
       
-      typedef typename std::mt19937_64::result_type local_seed_type;
-      
-      const uword n_threads = uword( mp_thread_limit::get() );
-      
-      std::vector< std::mt19937_64                  > engine(n_threads);
-      std::vector< std::normal_distribution<double> >  distr(n_threads);
-      
-      for(uword t=0; t < n_threads; ++t)
-        {
-        std::mt19937_64& t_engine = engine[t];
-        
-        t_engine.seed( local_seed_type(t) + local_seed_type(arma_rng::randi<local_seed_type>()) );
-        }
-      
-      const uword chunk_size = N / n_threads;
-      
-      #pragma omp parallel for schedule(static) num_threads(int(n_threads))
-      for(uword t=0; t < n_threads; ++t)
-        {
-        const uword start = (t+0) * chunk_size;
-        const uword endp1 = (t+1) * chunk_size;
-        
-        std::mt19937_64&                  t_engine = engine[t];
-        std::normal_distribution<double>& t_distr  =  distr[t];
-        
-        for(uword i=start; i < endp1; ++i)
-          {
-          const T val1 = T( t_distr(t_engine) );
-          const T val2 = T( t_distr(t_engine) );
-          
-          mem[i] = std::complex<T>(val1, val2);
-          }
-        }
-      
-      std::mt19937_64&                  t0_engine = engine[0];
-      std::normal_distribution<double>& t0_distr  =  distr[0];
-      
-      for(uword i=(n_threads*chunk_size); i < N; ++i)
-        {
-        const T val1 = T( t0_distr(t0_engine) );
-        const T val2 = T( t0_distr(t0_engine) );
-        
-        mem[i] = std::complex<T>(val1, val2);
-        }
+      mem[i] = std::complex<T>( ((val.real() * sd) + mu), ((val.imag() * sd) + mu) );
       }
-    #else
-      {
-      arma_rng::randn< std::complex<T> >::fill_simple(mem, N);
-      }
-    #endif
     }
   };
 
@@ -756,13 +1008,19 @@ struct arma_rng::randg
   inline
   static
   void
-  fill_simple(eT* mem, const uword N, const double a, const double b)
+  fill(eT* mem, const uword N, const double a, const double b)
     {
-    #if defined(ARMA_USE_EXTERN_RNG)
+    #if defined(ARMA_USE_CXX11_RNG)
       {
       std::gamma_distribution<double> local_g_distr(a,b);
       
-      for(uword i=0; i<N; ++i)  { mem[i] = eT(local_g_distr(mt19937_64_instance)); }
+      std::mt19937_64& producer = arma_rng::get_producer();
+      
+      arma_rng::lock_producer();
+      
+      for(uword i=0; i<N; ++i)  { mem[i] = eT(local_g_distr(producer)); }
+      
+      arma_rng::unlock_producer();
       }
     #else
       {
@@ -777,64 +1035,6 @@ struct arma_rng::randg
       }
     #endif
     }
-  
-  
-  inline
-  static
-  void
-  fill(eT* mem, const uword N, const double a, const double b)
-    {
-    #if defined(ARMA_USE_OPENMP)
-      {
-      if((N < 512) || omp_in_parallel())  { arma_rng::randg<eT>::fill_simple(mem, N, a, b); return; }
-      
-      typedef std::mt19937_64                  motor_type;
-      typedef std::mt19937_64::result_type      ovum_type;
-      typedef std::gamma_distribution<double>  distr_type;
-      
-      const uword n_threads = uword( mp_thread_limit::get() );
-      
-      std::vector<motor_type> g_motor(n_threads);
-      std::vector<distr_type> g_distr(n_threads);
-      
-      const distr_type g_distr_base(a,b);
-      
-      for(uword t=0; t < n_threads; ++t)
-        {
-        motor_type& g_motor_t = g_motor[t];
-        distr_type& g_distr_t = g_distr[t];
-        
-        g_motor_t.seed( ovum_type(t) + ovum_type(arma_rng::randi<ovum_type>()) );
-        
-        g_distr_t.param( g_distr_base.param() );
-        }
-      
-      const uword chunk_size = N / n_threads;
-      
-      #pragma omp parallel for schedule(static) num_threads(int(n_threads))
-      for(uword t=0; t < n_threads; ++t)
-        {
-        const uword start = (t+0) * chunk_size;
-        const uword endp1 = (t+1) * chunk_size;
-        
-        motor_type& g_motor_t = g_motor[t];
-        distr_type& g_distr_t = g_distr[t];
-        
-        for(uword i=start; i < endp1; ++i)  { mem[i] = eT( g_distr_t(g_motor_t)); }
-        }
-      
-      motor_type& g_motor_0 = g_motor[0];
-      distr_type& g_distr_0 = g_distr[0];
-      
-      for(uword i=(n_threads*chunk_size); i < N; ++i)  { mem[i] = eT( g_distr_0(g_motor_0)); }
-      }
-    #else
-      {
-      arma_rng::randg<eT>::fill_simple(mem, N, a, b);
-      }
-    #endif
-    }
-  
   };
 
 
