@@ -73,7 +73,7 @@ sp_auxlib::eigs_sym(Col<eT>& eigval, Mat<eT>& eigvec, const SpBase<eT, T1>& X, c
     if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "eigs_sym(): given matrix is not hermitian"); }
     }
   
-  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+  if(arma_config::check_nonfinite && U.M.internal_has_nonfinite())
     {
     arma_debug_warn_level(3, "eigs_sym(): detected non-finite elements");
     return false;
@@ -126,7 +126,7 @@ sp_auxlib::eigs_sym(Col<eT>& eigval, Mat<eT>& eigvec, const SpBase<eT, T1>& X, c
     if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "eigs_sym(): given matrix is not hermitian"); }
     }
   
-  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+  if(arma_config::check_nonfinite && U.M.internal_has_nonfinite())
     {
     arma_debug_warn_level(3, "eigs_sym(): detected non-finite elements");
     return false;
@@ -479,7 +479,9 @@ sp_auxlib::eigs_sym_arpack(Col<eT>& eigval, Mat<eT>& eigvec, const SpMat<eT>& X,
       }
     else
       {
-      run_aupd_plain(n_eigvals, which, X, true /* sym, not gen */, n, tol, maxiter, resid, ncv, v, ldv, iparam, ipntr, workd, workl, lworkl, rwork, info);
+      const SpMat<eT> Xst = X.st();
+      
+      run_aupd_plain(n_eigvals, which, X, Xst, true /* sym, not gen */, n, tol, maxiter, resid, ncv, v, ldv, iparam, ipntr, workd, workl, lworkl, rwork, info);
       }
     
     if(info != 0)  { return false; }
@@ -491,13 +493,11 @@ sp_auxlib::eigs_sym_arpack(Col<eT>& eigval, Mat<eT>& eigvec, const SpMat<eT>& X,
     char howmny = 'A';
     char bmat   = 'I'; // We are considering the standard eigenvalue problem.
     
-    podarray<blas_int> select(ncv); // Logical array of dimension NCV.
+    podarray<blas_int> select(ncv, arma_zeros_indicator()); // Logical array of dimension NCV.
     blas_int ldz = n;
     
-    select.zeros();
-    
     // seupd() will output directly into the eigval and eigvec objects.
-    eigval.zeros(n_eigvals);
+    eigval.zeros(   n_eigvals);
     eigvec.zeros(n, n_eigvals);
     
     arpack::seupd(&rvec, &howmny, select.memptr(), eigval.memptr(), eigvec.memptr(), &ldz, (eT*) &sigma, &bmat, &n, which, &nev, &tol, resid.memptr(), &ncv, v.memptr(), &ldv, iparam.memptr(), ipntr.memptr(), workd.memptr(), workl.memptr(), &lworkl, &info);
@@ -536,7 +536,7 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
-  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+  if(arma_config::check_nonfinite && U.M.internal_has_nonfinite())
     {
     arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
     return false;
@@ -583,7 +583,7 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
-  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+  if(arma_config::check_nonfinite && U.M.internal_has_nonfinite())
     {
     arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
     return false;
@@ -865,7 +865,9 @@ sp_auxlib::eigs_gen_arpack(Col< std::complex<T> >& eigval, Mat< std::complex<T>
       }
     else
       {
-      run_aupd_plain(n_eigvals, which, X, false /* gen, not sym */, n, tol, maxiter, resid, ncv, v, ldv, iparam, ipntr, workd, workl, lworkl, rwork, info);
+      const SpMat<T> Xst = X.st();
+      
+      run_aupd_plain(n_eigvals, which, X, Xst, false /* gen, not sym */, n, tol, maxiter, resid, ncv, v, ldv, iparam, ipntr, workd, workl, lworkl, rwork, info);
       }
     
     if(info != 0)  { return false; }
@@ -877,18 +879,13 @@ sp_auxlib::eigs_gen_arpack(Col< std::complex<T> >& eigval, Mat< std::complex<T>
     char howmny = 'A';
     char bmat   = 'I'; // We are considering the standard eigenvalue problem.
     
-    podarray<blas_int> select(ncv);      // logical array of dimension NCV
-    podarray<T>        dr(nev + 1);      // real array of dimension NEV + 1
-    podarray<T>        di(nev + 1);      // real array of dimension NEV + 1
-    podarray<T>        z(n * (nev + 1)); // real N by NEV array if HOWMNY = 'A'
-    blas_int ldz = n;
-    podarray<T>        workev(3 * ncv);
+    podarray<blas_int> select(ncv,          arma_zeros_indicator());  // logical array of dimension NCV
+    podarray<T>            dr(nev + 1,      arma_zeros_indicator());  // real array of dimension NEV + 1
+    podarray<T>            di(nev + 1,      arma_zeros_indicator());  // real array of dimension NEV + 1
+    podarray<T>            z(n * (nev + 1), arma_zeros_indicator());  // real N by NEV array if HOWMNY = 'A'
+    podarray<T>       workev(3 * ncv,       arma_zeros_indicator());
     
-    select.zeros();
-    dr.zeros();
-    di.zeros();
-    z.zeros();
-    workev.zeros();
+    blas_int ldz = n;
     
     arpack::neupd(&rvec, &howmny, select.memptr(), dr.memptr(), di.memptr(), z.memptr(), &ldz, (T*) &sigmar, (T*) &sigmai, workev.memptr(), &bmat, &n, which, &nev, &tol, resid.memptr(), &ncv, v.memptr(), &ldv, iparam.memptr(), ipntr.memptr(), workd.memptr(), workl.memptr(), &lworkl, rwork.memptr(), &info);
     
@@ -968,7 +965,7 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
-  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+  if(arma_config::check_nonfinite && U.M.internal_has_nonfinite())
     {
     arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
     return false;
@@ -993,7 +990,7 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
-  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+  if(arma_config::check_nonfinite && U.M.internal_has_nonfinite())
     {
     arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
     return false;
@@ -1112,7 +1109,9 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
       }
     else
       {
-      run_aupd_plain(n_eigvals, which, X, false /* gen, not sym */, n, tol, maxiter, resid, ncv, v, ldv, iparam, ipntr, workd, workl, lworkl, rwork, info);
+      const SpMat< std::complex<T> > Xst = X.st();
+      
+      run_aupd_plain(n_eigvals, which, X, Xst, false /* gen, not sym */, n, tol, maxiter, resid, ncv, v, ldv, iparam, ipntr, workd, workl, lworkl, rwork, info);
       }
     
     if(info != 0)  { return false; }
@@ -1124,16 +1123,12 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
     char howmny = 'A';
     char bmat   = 'I'; // We are considering the standard eigenvalue problem.
     
-    podarray<blas_int>        select(ncv); // logical array of dimension NCV
-    podarray<std::complex<T>> d(nev + 1);  // complex array of dimension NEV + 1
-    podarray<std::complex<T>> z(n * nev);  // complex N by NEV array if HOWMNY = 'A'
-    blas_int ldz = n;
-    podarray<std::complex<T>> workev(2 * ncv);
+    podarray<blas_int>        select(ncv,     arma_zeros_indicator());  // logical array of dimension NCV
+    podarray<std::complex<T>>      d(nev + 1, arma_zeros_indicator());  // complex array of dimension NEV + 1
+    podarray<std::complex<T>>      z(n * nev, arma_zeros_indicator());  // complex N by NEV array if HOWMNY = 'A'
+    podarray<std::complex<T>> workev(2 * ncv, arma_zeros_indicator());
     
-    select.zeros();
-    d.zeros();
-    z.zeros();
-    workev.zeros();
+    blas_int ldz = n;
     
     // Prepare the outputs; neupd() will write directly to them.
     eigval.zeros(n_eigvals);
@@ -1156,7 +1151,7 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
     arma_ignore(form_val);
     arma_ignore(sigma);
     arma_ignore(opts);
-
+    
     arma_stop_logic_error("eigs_gen(): use of ARPACK must be enabled for decomposition of complex matrices");
     return false;
     }
@@ -1201,7 +1196,7 @@ sp_auxlib::spsolve_simple(Mat<typename T1::elem_type>& X, const SpBase<typename
     
     if(A.n_nonzero == uword(0))  { X.soft_reset(); return false; }
     
-    if(arma_config::check_nonfinite && (A.has_nonfinite() || X.has_nonfinite()))
+    if(arma_config::check_nonfinite && (A.internal_has_nonfinite() || X.internal_has_nonfinite()))
       {
       arma_debug_warn_level(3, "spsolve(): detected non-finite elements");
       return false;
@@ -1258,7 +1253,7 @@ sp_auxlib::spsolve_simple(Mat<typename T1::elem_type>& X, const SpBase<typename
     else
     if(info > int(A.n_cols))
       {
-      arma_debug_warn_level(1, "spsolve(): memory allocation failure: could not allocate ", (info - int(A.n_cols)), " bytes");
+      arma_debug_warn_level(1, "spsolve(): memory allocation failure");
       }
     else
     if(info < 0)
@@ -1326,7 +1321,7 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
     
     if(A.n_nonzero == uword(0))  { X.soft_reset(); return false; }
     
-    if(arma_config::check_nonfinite && (A.has_nonfinite() || X.has_nonfinite()))
+    if(arma_config::check_nonfinite && (A.internal_has_nonfinite() || B.internal_has_nonfinite()))
       {
       arma_debug_warn_level(3, "spsolve(): detected non-finite elements");
       return false;
@@ -1445,79 +1440,6 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
 
 
 
-// template<typename T1>
-// inline
-// typename T1::pod_type
-// sp_auxlib::rcond(const SpBase<typename T1::elem_type, T1>& A_expr)
-//   {
-//   arma_extra_debug_sigprint();
-//   
-//   #if defined(ARMA_USE_SUPERLU)
-//     {
-//     typedef typename T1::pod_type   T;
-//     typedef typename T1::elem_type eT;
-//     
-//     const unwrap_spmat<T1> tmp1(A_expr.get_ref());
-//     const SpMat<eT>& A =   tmp1.M;
-//     
-//     arma_debug_check( (A.is_square() == false), "rcond(): matrix must be square sized" );
-//     
-//     superlu_opts superlu_opts_default;
-//     superlu::superlu_options_t options;
-//     sp_auxlib::set_superlu_opts(options, superlu_opts_default);
-//     int lwork = 0;
-//     
-//     superlu::GlobalLU_t Glu;
-//     arrayops::fill_zeros(reinterpret_cast<char*>(&Glu), sizeof(superlu::GlobalLU_t));
-//     
-//     superlu_supermatrix_wrangler a;
-//     superlu_supermatrix_wrangler aC;
-//     
-//     const bool status_a = sp_auxlib::copy_to_supermatrix(a.get_ref(), A);
-//     
-//     if(status_a == false)  { arma_stop_runtime_error("rcond(): could not construct SuperLU matrix"); return T(0); }
-//     
-//     superlu_supermatrix_wrangler l;
-//     superlu_supermatrix_wrangler u;
-//     
-//     superlu_array_wrangler<int> perm_c(A.n_cols+1);  // paranoia: increase array length by 1
-//     superlu_array_wrangler<int> perm_r(A.n_rows+1);
-//     superlu_array_wrangler<int>  etree(A.n_cols+1);
-//     
-//     superlu_stat_wrangler stat;
-//     
-//     int panel_size = superlu::sp_ispec_environ(1);
-//     int relax      = superlu::sp_ispec_environ(2);
-//     int slu_info   = 0; // Return code.
-//     
-//     arma_extra_debug_print("superlu::gstrf()");
-//     superlu::get_permutation_c(options.ColPerm, a.get_ptr(), perm_c.get_ptr());
-//     superlu::sp_preorder_mat(&options, a.get_ptr(), perm_c.get_ptr(), etree.get_ptr(), aC.get_ptr());
-//     superlu::gstrf<T>(&options, aC.get_ptr(), relax, panel_size, etree.get_ptr(), NULL, lwork, perm_c.get_ptr(), perm_r.get_ptr(), l.get_ptr(), u.get_ptr(), &Glu, stat.get_ptr(), &slu_info);
-//     
-//     if(slu_info != 0)  { return T(0); }
-//     
-//     T a_norm_val = sp_auxlib::norm1<eT>(a.get_ptr());
-//     T a_rcond    = sp_auxlib::lu_rcond<eT>(l.get_ptr(), u.get_ptr(), a_norm_val);
-//     
-//     if(arma_isnan(a_rcond))  { return T(0); }
-//     
-//     return a_rcond;
-//     }
-//   #else
-//     {
-//     typename T1::pod_type T;
-//     
-//     arma_ignore(A_expr);
-//     arma_stop_logic_error("rcond(): use of SuperLU must be enabled");
-//     
-//     return T(0);
-//     }
-//   #endif
-//   }
-
-
-
 #if defined(ARMA_USE_SUPERLU)
   
   template<typename eT>
@@ -1529,6 +1451,7 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
     
     char norm_id = '1';
     
+    arma_extra_debug_print("superlu::langs()");
     return superlu::langs<eT>(&norm_id, A);
     }
   
@@ -1549,6 +1472,7 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
     
     superlu_stat_wrangler stat;
     
+    arma_extra_debug_print("superlu::gscon()");
     superlu::gscon<eT>(&norm_id, L, U, norm_val, &rcond_out, stat.get_ptr(), &info);
     
     return (info == 0) ? T(rcond_out) : T(0);
@@ -1718,10 +1642,10 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
     
     const uword out_n_nonzero = A.n_nonzero - n_nonzero_diag_old + n_nonzero_diag_new;
     
-    arma_extra_debug_print( arma_str::format("A.n_nonzero:        %d") % A.n_nonzero        );
-    arma_extra_debug_print( arma_str::format("n_nonzero_diag_old: %d") % n_nonzero_diag_old );
-    arma_extra_debug_print( arma_str::format("n_nonzero_diag_new: %d") % n_nonzero_diag_new );
-    arma_extra_debug_print( arma_str::format("out_n_nonzero:      %d") % out_n_nonzero      );
+    arma_extra_debug_print( arma_str::format("A.n_nonzero:        %u") % A.n_nonzero        );
+    arma_extra_debug_print( arma_str::format("n_nonzero_diag_old: %u") % n_nonzero_diag_old );
+    arma_extra_debug_print( arma_str::format("n_nonzero_diag_new: %u") % n_nonzero_diag_new );
+    arma_extra_debug_print( arma_str::format("out_n_nonzero:      %u") % out_n_nonzero      );
     
     nc->nnz    = out_n_nonzero;
     nc->nzval  = (void*)          superlu::malloc(sizeof(eT)             * out_n_nonzero );
@@ -1813,7 +1737,7 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
       nc->colptr[j + 1] = superlu::int_t(nc->colptr[j] + nnz_col);
       }
     
-    arma_extra_debug_print( arma_str::format("count: %d") % count );
+    arma_extra_debug_print( arma_str::format("count: %u") % count );
     
     arma_check( (count != out_n_nonzero), "internal error: sp_auxlib::copy_to_supermatrix_with_shift(): count != out_n_nonzero" );
     
@@ -1844,6 +1768,9 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
 //     arma_debug_check( (type_matched == false),      "copy_to_spmat(): type mismatch"  );
 //     arma_debug_check( (A.Mtype != superlu::SLU_GE), "copy_to_spmat(): unknown layout" );
 //     
+//     // NOTE: the l and u instances of SuperMatrix resulting from superlu::gstrf()
+//     // NOTE: do not have the superlu::SLU_GE layout
+//     
 //     const superlu::NCformat* nc = (const superlu::NCformat*)(A.Store);
 //     
 //     if(nc == nullptr)  { out.reset(); return; }
@@ -1862,6 +1789,8 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
 //     
 //     arrayops::convert(access::rwp(out.col_ptrs),    nc->colptr, A_n_cols+1 );
 //     arrayops::convert(access::rwp(out.row_indices), nc->rowind, A_n_nonzero);
+//     
+//     out.remove_zeros();  // in case SuperLU has bugs and stores zeros in sparse matrices
 //     }
   
   
@@ -1970,7 +1899,7 @@ void
 sp_auxlib::run_aupd_plain
   (
   const uword n_eigvals, char* which, 
-  const SpMat<T>& X, const bool sym,
+  const SpMat<T>& X, const SpMat<T>& Xst, const bool sym,
   blas_int& n, eT& tol, blas_int& maxiter,
   podarray<T>& resid, blas_int& ncv, podarray<T>& v, blas_int& ldv,
   podarray<blas_int>& iparam, podarray<blas_int>& ipntr,
@@ -1992,7 +1921,9 @@ sp_auxlib::run_aupd_plain
     n = X.n_rows; // The size of the matrix (should already be set outside).
     blas_int nev = n_eigvals;
     
-    resid.zeros(n);
+    // resid.zeros(n);
+    eigs_randu_filler<T> randu_filler;
+    randu_filler.fill(resid, n);  // use deterministic starting point
     
     // Two contraints on NCV: (NCV > NEV) for sym problems or
     // (NCV > NEV + 2) for gen problems and (NCV <= N)
@@ -2027,7 +1958,8 @@ sp_auxlib::run_aupd_plain
     // Real work array of length lworkl.
     workl.zeros(lworkl);
     
-    info = 0; // Set to 0 initially to use random initial vector.
+    // info = 0; // resid to be filled with random values by ARPACK (non-deterministic)
+    info = 1; // resid is already filled with random values (deterministic)
     
     // All the parameters have been set or created.  Time to loop a lot.
     while(ido != 99)
@@ -2055,30 +1987,46 @@ sp_auxlib::run_aupd_plain
           // where x is of length n and starts at workd(ipntr(0)), and y is of
           // length n and starts at workd(ipntr(1)).
           
-          // operator*(sp_mat, vec) doesn't properly put the result into the
-          // right place so we'll just reimplement it here for now...
+          // // OLD METHOD
+          // 
+          // // operator*(sp_mat, vec) doesn't properly put the result into the
+          // // right place so we'll just reimplement it here for now...
+          // 
+          // // Set the output to point at the right memory.  We have to subtract
+          // // one from FORTRAN pointers...
+          // Col<T> out(workd.memptr() + ipntr(1) - 1, n, false /* don't copy */);
+          // // Set the input to point at the right memory.
+          // Col<T>  in(workd.memptr() + ipntr(0) - 1, n, false /* don't copy */);
+          // 
+          // out.zeros();
+          // 
+          //       T* out_mem = out.memptr();
+          // const T*  in_mem =  in.memptr();
+          // 
+          // typename SpMat<T>::const_iterator X_it = X.begin();
+          // 
+          // const uword X_nnz = X.n_nonzero;
+          // 
+          // for(uword count=0; count < X_nnz; ++count, ++X_it)
+          //   {
+          //   const eT    X_it_val = (*X_it);
+          //   const uword X_it_row = X_it.row();
+          //   const uword X_it_col = X_it.col();
+          //   
+          //   out_mem[X_it_row] += X_it_val * in_mem[X_it_col];
+          //   }
+          // 
+          // // No need to modify memory further since it was all done in-place.
           
-          // Set the output to point at the right memory.  We have to subtract
-          // one from FORTRAN pointers...
-          Col<T> out(workd.memptr() + ipntr(1) - 1, n, false /* don't copy */);
-          // Set the input to point at the right memory.
-          Col<T> in(workd.memptr() + ipntr(0) - 1, n, false /* don't copy */);
           
-          out.zeros();
+          // NEW METHOD
+          // 
+          // both operator*(rowvec, sp_mat) and operator*(sp_mat, colvec) can now write to an existing object
           
-          typename SpMat<T>::const_iterator X_it     = X.begin();
-          typename SpMat<T>::const_iterator X_it_end = X.end();
+          Row<T> out(workd.memptr() + ipntr(1) - 1, n, false, true);
+          Row<T>  in(workd.memptr() + ipntr(0) - 1, n, false, true);
           
-          while(X_it != X_it_end)
-            {
-            const uword X_it_row = X_it.row();
-            const uword X_it_col = X_it.col();
-            
-            out[X_it_row] += (*X_it) * in[X_it_col];
-            ++X_it;
-            }
-          
-          // No need to modify memory further since it was all done in-place.
+          out = in * Xst;
           
           break;
           }
@@ -2164,7 +2112,9 @@ sp_auxlib::run_aupd_shiftinvert
     n = X.n_rows; // The size of the matrix (should already be set outside).
     blas_int nev = n_eigvals;
     
-    resid.zeros(n);
+    // resid.zeros(n);
+    eigs_randu_filler<T> randu_filler;
+    randu_filler.fill(resid, n);  // use deterministic starting point
     
     // Two contraints on NCV: (NCV > NEV) for sym problems or
     // (NCV > NEV + 2) for gen problems and (NCV <= N)
@@ -2202,7 +2152,8 @@ sp_auxlib::run_aupd_shiftinvert
     // Real work array of length lworkl.
     workl.zeros(lworkl);
     
-    info = 0; // Set to 0 initially to use random initial vector.
+    // info = 0; // resid to be filled with random values by ARPACK (non-deterministic)
+    info = 1; // resid is already filled with random values (deterministic)
     
     superlu_opts superlu_opts_default;
     superlu::superlu_options_t options;
@@ -2274,6 +2225,7 @@ sp_auxlib::run_aupd_shiftinvert
       return;
       }
     
+    // NOTE: potential problem with inconsistent/mismatched use of eT and T types
     eT x_norm_val = sp_auxlib::norm1<T>(x.get_ptr());
     eT x_rcond    = sp_auxlib::lu_rcond<T>(l.get_ptr(), u.get_ptr(), x_norm_val);
     
@@ -2310,9 +2262,6 @@ sp_auxlib::run_aupd_shiftinvert
           // where x is of length n and starts at workd(ipntr(0)), and y is of
           // length n and starts at workd(ipntr(1)).
           
-          // operator*(sp_mat, vec) doesn't properly put the result into the
-          // right place so we'll just reimplement it here for now...
-          
           // Set the output to point at the right memory.  We have to subtract
           // one from FORTRAN pointers...
           Col<T> out(workd.memptr() + ipntr(1) - 1, n, false /* don't copy */);
@@ -2496,6 +2445,85 @@ sp_auxlib::rudimentary_sym_check(const SpMat< std::complex<T> >& X)
 
 
 
+//
+
+
+
+template<typename eT>
+inline
+eigs_randu_filler<eT>::eigs_randu_filler()
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename std::mt19937_64::result_type local_seed_type;
+  
+  local_engine.seed(local_seed_type(123));
+  
+  typedef typename std::uniform_real_distribution<eT>::param_type local_param_type;
+  
+  local_u_distr.param(local_param_type(-1.0, +1.0));
+  }
+
+
+template<typename eT>
+inline
+void
+eigs_randu_filler<eT>::fill(podarray<eT>& X, const uword N)
+  {
+  arma_extra_debug_sigprint();
+  
+  X.set_size(N);
+  
+  eT* X_mem = X.memptr();
+  
+  for(uword i=0; i<N; ++i)  { X_mem[i] = eT( local_u_distr(local_engine) ); }
+  }
+
+
+template<typename T>
+inline
+eigs_randu_filler< std::complex<T> >::eigs_randu_filler()
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename std::mt19937_64::result_type local_seed_type;
+  
+  local_engine.seed(local_seed_type(123));
+  
+  typedef typename std::uniform_real_distribution<T>::param_type local_param_type;
+  
+  local_u_distr.param(local_param_type(-1.0, +1.0));
+  }
+
+
+template<typename T>
+inline
+void
+eigs_randu_filler< std::complex<T> >::fill(podarray< std::complex<T> >& X, const uword N)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename std::complex<T> eT;
+  
+  X.set_size(N);
+  
+  eT* X_mem = X.memptr();
+  
+  for(uword i=0; i<N; ++i)
+    {
+    eT& X_mem_i = X_mem[i];
+    
+    X_mem_i.real( T(local_u_distr(local_engine)) );
+    X_mem_i.imag( T(local_u_distr(local_engine)) );
+    }
+  }
+
+
+
+//
+
+
+
 #if defined(ARMA_USE_SUPERLU)
 
 inline
@@ -2581,19 +2609,36 @@ superlu_array_wrangler<eT>::~superlu_array_wrangler()
   {
   arma_extra_debug_sigprint_this(this);
   
-  if(mem != nullptr)
-    {
-    superlu::free(mem);
-    mem = nullptr;
-    }
+  (*this).reset();
+  }
+
+template<typename eT>
+inline
+superlu_array_wrangler<eT>::superlu_array_wrangler()
+  : mem(nullptr)
+  {
+  arma_extra_debug_sigprint_this(this);
   }
 
 template<typename eT>
 inline
 superlu_array_wrangler<eT>::superlu_array_wrangler(const uword n_elem)
+  : mem(nullptr)
   {
   arma_extra_debug_sigprint_this(this);
   
+  (*this).set_size(n_elem);
+  }
+
+template<typename eT>
+inline
+void
+superlu_array_wrangler<eT>::set_size(const uword n_elem)
+  {
+  arma_extra_debug_sigprint();
+  
+  if(mem != nullptr)  { (*this).reset(); }
+  
   mem = (eT*)(superlu::malloc(n_elem * sizeof(eT)));
   
   arma_check_bad_alloc( (mem == nullptr), "superlu::malloc(): out of memory" );
@@ -2601,6 +2646,20 @@ superlu_array_wrangler<eT>::superlu_array_wrangler(const uword n_elem)
   arrayops::fill_zeros(mem, n_elem);
   }
 
+template<typename eT>
+inline
+void
+superlu_array_wrangler<eT>::reset()
+  {
+  arma_extra_debug_sigprint();
+  
+  if(mem != nullptr)
+    {
+    superlu::free(mem);
+    mem = nullptr;
+    }
+  }
+
 template<typename eT>
 inline
 eT*
@@ -2609,6 +2668,146 @@ superlu_array_wrangler<eT>::get_ptr()
   return mem;
   }
 
+
+//
+
+
+template<typename eT>
+inline
+superlu_worker<eT>::~superlu_worker()
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  if(l != nullptr)  { delete l; l = nullptr; }
+  if(u != nullptr)  { delete u; u = nullptr; }
+  }
+
+
+template<typename eT>
+inline
+superlu_worker<eT>::superlu_worker()
+  {
+  arma_extra_debug_sigprint_this(this);
+  }
+
+
+template<typename eT>
+inline
+bool
+superlu_worker<eT>::factorise(typename get_pod_type<eT>::result& out_rcond, const SpMat<eT>& A, const superlu_opts& user_opts)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename get_pod_type<eT>::result T;
+  
+  factorisation_valid = false;
+  
+  if(l != nullptr)  { delete l; l = nullptr; }
+  if(u != nullptr)  { delete u; u = nullptr; }
+  
+  l = new(std::nothrow) superlu_supermatrix_wrangler;
+  u = new(std::nothrow) superlu_supermatrix_wrangler;
+  
+  if( (l == nullptr) || (u == nullptr) )
+    {
+    arma_debug_warn_level(3, "superlu_worker()::factorise(): could not construct SuperLU matrix");
+    return false;
+    }
+  
+  superlu_supermatrix_wrangler& l_ref = (*l);
+  superlu_supermatrix_wrangler& u_ref = (*u);
+  
+  superlu::superlu_options_t options;
+  sp_auxlib::set_superlu_opts(options, user_opts);
+  
+  superlu_supermatrix_wrangler AA;
+  superlu_supermatrix_wrangler AAc;
+  
+  const bool status_AA = sp_auxlib::copy_to_supermatrix(AA.get_ref(), A);
+  
+  if(status_AA == false)
+    {
+    arma_debug_warn_level(3, "superlu_worker()::factorise(): could not construct SuperLU matrix");
+    return false;
+    }
+  
+  (*this).perm_c.set_size(A.n_cols+1);  // paranoia: increase array length by 1
+  (*this).perm_r.set_size(A.n_rows+1);
+  
+  superlu_array_wrangler<int> etree(A.n_cols+1);
+  
+  superlu::GlobalLU_t Glu;
+  arrayops::fill_zeros(reinterpret_cast<char*>(&Glu), sizeof(superlu::GlobalLU_t));
+  
+  int panel_size = superlu::sp_ispec_environ(1);
+  int relax      = superlu::sp_ispec_environ(2);
+  int lwork      = 0;
+  int info       = 0;
+  
+  arma_extra_debug_print("superlu::superlu::get_permutation_c()");
+  superlu::get_permutation_c(options.ColPerm, AA.get_ptr(), perm_c.get_ptr());
+  
+  arma_extra_debug_print("superlu::superlu::sp_preorder_mat()");
+  superlu::sp_preorder_mat(&options, AA.get_ptr(), perm_c.get_ptr(), etree.get_ptr(), AAc.get_ptr());
+  
+  arma_extra_debug_print("superlu::gstrf()");
+  superlu::gstrf<eT>(&options, AAc.get_ptr(), relax, panel_size, etree.get_ptr(), NULL, lwork, perm_c.get_ptr(), perm_r.get_ptr(), l_ref.get_ptr(), u_ref.get_ptr(), &Glu, stat.get_ptr(), &info);
+  
+  if(info != 0)
+    {
+    arma_debug_warn_level(3, "superlu_worker()::factorise(): LU factorisation failed");
+    return false;
+    }
+  
+  const T AA_norm  = sp_auxlib::norm1<T>(AA.get_ptr());
+  const T AA_rcond = sp_auxlib::lu_rcond<eT>(l_ref.get_ptr(), u_ref.get_ptr(), AA_norm);
+  
+  out_rcond = AA_rcond;
+  
+  if(arma_isnan(AA_rcond))  { return false; }
+  // if(AA_rcond == T(0))      { return false; }
+  
+  factorisation_valid = true;
+  
+  return true;
+  }
+
+
+template<typename eT>
+inline
+bool
+superlu_worker<eT>::solve(Mat<eT>& X, const Mat<eT>& B)
+  {
+  arma_extra_debug_sigprint();
+  
+  if(factorisation_valid == false)        { return false; }
+  if( (l == nullptr) || (u == nullptr) )  { return false; }
+  
+  superlu_supermatrix_wrangler& l_ref = (*l);
+  superlu_supermatrix_wrangler& u_ref = (*u);
+  
+  X = B;
+  
+  superlu_supermatrix_wrangler XX;
+  
+  const bool status_XX = sp_auxlib::wrap_to_supermatrix(XX.get_ref(), X);
+  
+  if(status_XX == false)
+    {
+    arma_debug_warn_level(3, "superlu_worker()::solve(): could not construct SuperLU matrix");
+    return false;
+    }
+  
+  superlu::trans_t trans = superlu::NOTRANS;
+  int              info  = 0;
+  
+  arma_extra_debug_print("superlu::gstrs()");
+  superlu::gstrs<eT>(trans, l_ref.get_ptr(), u_ref.get_ptr(), perm_c.get_ptr(), perm_r.get_ptr(), XX.get_ptr(), stat.get_ptr(), &info);
+  
+  return (info == 0);
+  }
+
+
 #endif
 
 
