@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2011-2017 Ryan Curtin (http://www.ratml.org/)
 // Copyright 2017 National ICT Australia (NICTA)
 //
@@ -65,10 +67,10 @@ TEST_CASE("fn_eigs_gen_odd_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.1) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.1) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.1) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.1) );
         }
       }
     }
@@ -127,10 +129,10 @@ TEST_CASE("fn_eigs_gen_even_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -190,10 +192,10 @@ TEST_CASE("fn_eigs_gen_even_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -254,10 +256,10 @@ TEST_CASE("fn_eigs_gen_odd_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.1) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.1) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.1) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.1) );
         }
       }
     }
@@ -318,10 +320,10 @@ TEST_CASE("fn_eigs_gen_even_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -383,10 +385,10 @@ TEST_CASE("fn_eigs_gen_even_sigma_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -446,10 +448,10 @@ TEST_CASE("fn_eigs_gen_odd_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.1) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.1) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.1) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.1) );
         }
       }
     }
@@ -509,10 +511,10 @@ TEST_CASE("fn_eigs_gen_even_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -573,10 +575,10 @@ TEST_CASE("fn_eigs_gen_even_sm_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -636,10 +638,10 @@ TEST_CASE("fn_eigs_gen_odd_float_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.001) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.001) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -699,10 +701,10 @@ TEST_CASE("fn_eigs_gen_even_float_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -763,10 +765,10 @@ TEST_CASE("fn_eigs_gen_even_float_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -828,10 +830,10 @@ TEST_CASE("fn_eigs_gen_odd_float_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.001) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.001) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -893,10 +895,10 @@ TEST_CASE("fn_eigs_gen_even_float_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -959,10 +961,10 @@ TEST_CASE("fn_eigs_gen_even_float_sigma_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1023,10 +1025,10 @@ TEST_CASE("fn_eigs_gen_odd_float_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.001) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.001) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1087,10 +1089,10 @@ TEST_CASE("fn_eigs_gen_even_float_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1152,10 +1154,10 @@ TEST_CASE("fn_eigs_gen_even_float_sm_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1211,10 +1213,10 @@ TEST_CASE("fn_eigs_gen_odd_complex_float_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1270,10 +1272,10 @@ TEST_CASE("fn_eigs_gen_even_complex_float_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1330,10 +1332,10 @@ TEST_CASE("fn_eigs_gen_even_complex_float_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1394,10 +1396,10 @@ TEST_CASE("fn_eigs_gen_odd_complex_float_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1458,10 +1460,10 @@ TEST_CASE("fn_eigs_gen_even_complex_float_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1523,10 +1525,10 @@ TEST_CASE("fn_eigs_gen_even_complex_float_sigma_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1586,10 +1588,10 @@ TEST_CASE("fn_eigs_gen_odd_complex_float_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1649,10 +1651,10 @@ TEST_CASE("fn_eigs_gen_even_complex_float_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1713,10 +1715,10 @@ TEST_CASE("fn_eigs_gen_even_complex_float_sm_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1772,10 +1774,10 @@ TEST_CASE("eigs_gen_odd_complex_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(size_t j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1831,10 +1833,10 @@ TEST_CASE("fn_eigs_gen_even_complex_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1891,10 +1893,10 @@ TEST_CASE("fn_eigs_gen_even_complex_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -1955,10 +1957,10 @@ TEST_CASE("eigs_gen_odd_complex_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(size_t j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -2019,10 +2021,10 @@ TEST_CASE("fn_eigs_gen_even_complex_sigma_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -2084,10 +2086,10 @@ TEST_CASE("fn_eigs_gen_even_complex_sigma_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -2144,10 +2146,10 @@ TEST_CASE("eigs_gen_odd_complex_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(size_t j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -2204,10 +2206,10 @@ TEST_CASE("fn_eigs_gen_even_complex_sm_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
@@ -2265,10 +2267,10 @@ TEST_CASE("fn_eigs_gen_even_complex_sm_opts_test")
 
       REQUIRE( dense_eval != n_rows + 1 );
 
-      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).epsilon(0.01) );
+      REQUIRE( std::abs(sp_eigval(i)) == Approx(std::abs(eigval(dense_eval))).margin(0.01) );
       for(uword j = 0; j < n_rows; ++j)
         {
-        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).epsilon(0.01) );
+        REQUIRE( std::abs(sp_eigvec(j, i)) == Approx(std::abs(eigvec(j, dense_eval))).margin(0.01) );
         }
       }
     }
